# Java微服务架构师

## Java并发编程基础

### 并发安全

- 案例

	- 集合遍历问题（list）
	- 共享成员变量

- 线程安全的集合
- 线程安全概念
- 编码规约

	- 考虑代码健壮性
	- 常见场景敏感性

### 同步类容器（串行）

- 同步锁

	- Vector
	- HashTable

- 底层机制

	- 都是由JDK的Collections.synchronized***等工厂方法去创建实现的
	- 使用synchronized关键字对每个公用的方法都进行同步
	- 使用Object mutex对象锁使得每次只能有一个线程访问容器的状态

- 问题

	- 还可能出现（并发修改异常）
	- 并发处理性能差
	- 并发场景下，严重降低吞吐量
	-  锁竞争

###  并发类容器（异步）jdk1.5开始

- ConcurrentHashMap

	- 基本使用

		- size()

			- 消除伪共享（缓存行）
			- 分之思想
			- volatile long value

	- 基本原理

		- 内部结构使用段（Segment）来表示不同的部分，每一段其实就是一个小的HashTable升级版，它们有自己的锁，降低了锁的粒度
		- 只要多个修改操作发生在不同的段上，就可以并发进行，默认16段，所以最高支持16个线程的并发修改操作
		- 减小锁的粒度达到降低锁竞争的方式，并且代码中大多共享变量采用volatile关键字声明，目的是第一时间获取修改的内容，性能非常好。

- CopyOnWrite

	- 基本使用

		- jdk中常用的

			- CopyOnWriteArrayList
			- ConyOnWriteArraySet

		- 最佳使用场景

			- 读多写少，元素不能太多

	- 基本原理

		- 往一个容器添加元素时，不直接往当前的容器添加，而是将当前容器进行Copy，复制出一个新容器，然后新容器里添加元素，添加完元素之后，再将原容器的引用指向新的容器。
		- 采用读写分离的思想，读和写不同的容器，实现对CopyOnWrite容器进行并发的读，而不需要加锁，因为当前这个容器不会添加任何的元素。

- ConcurrentSkipListMap

	- 支持并发排序

### 队列

- (顶层接口Queue)

	- 并发队列（ConcurrentLinkedQueue）

		- 适应高并发场景的队列，通过无锁的方式，实现高并发状态下的高性能，通常性能好于BlockingQueue，是一个基于链表的无界线程安全队列
		- 该队列遵循先进先出原则，头是最先加入，尾是最后加入，该队列不允许null元素。
		- 基本使用

			- add()和offer()都是添加元素方法，在这里这两个方法没有任何区别
			- poll()和peek()都是取头元素节点，区别在于前者是会删除元素，后者不会

	- 阻塞队列（BlockingQueue）

		- 基本使用

			- offer(anObject) 将anObject加到BlockingQueue里，即如果BlockingQueue可以容纳则返回true，否则返回false (不会阻塞线程) 
			- offer(E e, long timeout ,TimeUint uint ) 设置等待时间，如果指定时间内，还不能往队列加入则返回失败。 
			- put() 把anObject加入BlockingQueue里，如果BlockingQueue没有空间，调用此方法的线程会被阻塞直到队列里有空间为止。
			- poll(long timeout，TimeUint uint) 从队列里取出一个队首元素，如果在指定时间内，队列一旦有元素可取，则立即返回队列中的元素，否则直到时间超时还没有元素可取，返回失败。
			- take()取出BlockiQueue里排在队首的元素，若队列为空，阻塞进入等待状态直到队列有新的元素被加入。
			- drainTo() 一次性从队列获取所有可用的元素（还可以指定获取的个数），通过该方法，可以提高获取元素的效率，不需要多次分批加锁或释放锁。

		- 常用实现

			- 基于数组的阻塞队列实现ArrayBlockingQueue 

				- 内部维护一个定长数组，缓存队列中的数据元素，其内部没有实现读写分离，也就意味着生产和消费不能完全并行。长度需要定义，可以指定先进先出或者先进后出，由于是定长所以也叫有界队列。
				- 子主题 2

			- LinkedBlockingDeque
			- PriorityBlockingQueue

				- 基于优先级的阻塞队列，出队列是依据优先级
				- 优先级的判断是通过构造函数传入的Compator对象来决定，也就是说传入的队列的元素必须实现Comparable接口，在实现PriorityBlockingQueue时，内部控制线程同步的锁采用的是公平锁，也一个无界队列。
				- 元素不能为基本类型

			- SynchronousQueue

				- 一种没有缓冲的阻塞队列
				- 生产者生产的数据直接会被消费者获取并消费

			- DelayQueue

				- 带有延迟时间的队列
				- 其中的元素只有指定延迟时间到了，才能够从队列中获取到该元素，DelayQueue的元素必须实现Delayed接口，该队列是一个无界队列，应用场景：对缓存超时的数据进行移除，任务超时处理，空闲链接超时关闭等。

	- 阻塞队列手写模拟

		- 拥有固定长度的装载元素的容器
		- 计数器统计容器的容量大小
		- 当队列里面没有元素的时候需要执行线程等待
		- 当队列元素已满的时候执行的线程也需要等待
		- 合理的线程间通信

### 锁

- 常见锁

	- 同步锁
	- Lock
	- LockSupport

- 子主题 2

### JUC

- AQS
- CAS
- 分类

## 分布式中间件

### 缓存组件Redis

## 微服务架构

### Spring Boot

## 高并发架构

## 高性能架构

## 项目架构设计实战

*XMind - Trial Version*
> 
